"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const buffer_xor_1 = __importDefault(require("buffer-xor"));
const crypto_1 = require("../crypto");
const __1 = require("..");
exports.NUMBER_OF_CHARACTERS = 32;
/**
 * Hasher generate a modified hash unique to a source and its signers, or recover the hash of the original data.
 * When applying it, it should prepend the crumbs to finalize the Crumbl.
 * Algorithm: N-32 last chars of the hex string of the hashed source + (32 last chars ^ padded/cut lexicographically sorted owners encrypted crumbs).toHex
 */
class Hasher {
    constructor(crumbs) {
        this.crumbs = crumbs;
    }
    /**
     * Build the hashered prefix for the Crumbl
     *
     * @param {string} source - the data source
     * @returns the hashered string
     */
    apply(source) {
        return __awaiter(this, void 0, void 0, function* () {
            const stringifiedHash = yield crypto_1.hash(source, crypto_1.DEFAULT_CRYPTO_HASH_ENGINE);
            const length = stringifiedHash.length;
            if (length < exports.NUMBER_OF_CHARACTERS) {
                const msg = 'wrong hash algorithm';
                throw new Error(msg);
            }
            const lastBytes = Buffer.from(stringifiedHash.substring(length - exports.NUMBER_OF_CHARACTERS), 'hex');
            const sortedOwnerCrumbs = this.crumbs.filter(crumb => crumb.index === 0).map(_ => _.encrypted.toString()).sort(); // eslint-disable-line @typescript-eslint/require-array-sort-compare
            if (sortedOwnerCrumbs.length < 1) {
                throw new Error('owner\'s crumbs not present');
            }
            const concat = Buffer.concat(sortedOwnerCrumbs.map(c => Buffer.from(c, 'base64')));
            const mask = buildMask(concat, exports.NUMBER_OF_CHARACTERS / 2);
            const xored = buffer_xor_1.default(lastBytes, mask);
            const xoredHex = xored.toString('hex');
            return stringifiedHash.substring(0, length - exports.NUMBER_OF_CHARACTERS) + xoredHex;
        });
    }
    /**
     * Recover the original hash
     *
     * @param {string} hashered - the hashered string
     * @returns the original hexadecimal representation of the hash of the source data
     */
    unapply(hashered) {
        if (hashered.length < exports.NUMBER_OF_CHARACTERS) {
            throw new Error('wrong hashered value');
        }
        const xoredHex = hashered.substring(hashered.length - exports.NUMBER_OF_CHARACTERS);
        const xored = Buffer.from(xoredHex, 'hex');
        const sortedOwnerCrumbs = this.crumbs.filter(crumb => crumb.index === 0).map(_ => _.encrypted.toString()).sort(); // eslint-disable-line @typescript-eslint/require-array-sort-compare
        if (sortedOwnerCrumbs.length < 1) {
            throw new Error('owner\'s crumbs not present');
        }
        const concat = Buffer.concat(sortedOwnerCrumbs.map(c => Buffer.from(c, 'base64')));
        const mask = buildMask(concat, exports.NUMBER_OF_CHARACTERS / 2);
        const lastBytes = buffer_xor_1.default(xored, mask);
        return hashered.substring(0, hashered.length - exports.NUMBER_OF_CHARACTERS) + lastBytes.toString('hex');
    }
}
exports.Hasher = Hasher;
const buildMask = (key, length) => {
    let mask = Buffer.alloc(0);
    if (key.length === 0) {
        return mask;
    }
    if (key.length >= length) {
        return key.slice(0, length);
    }
    const [quotient, remainder] = __1.euclideanDivision(length, key.length);
    for (let i = 0; i < quotient; i++) {
        mask = Buffer.concat([mask, key]);
    }
    return Buffer.concat([mask, key.slice(0, remainder)]);
};
