"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const feistel_cipher_1 = require("feistel-cipher");
const Collector_1 = require("../Decrypter/Collector");
const Crumb_1 = require("../Encrypter/Crumb");
const Decrypter_1 = require("../Decrypter");
const crypto_1 = require("../crypto");
const Obfuscator_1 = require("../Obfuscator");
const Crumbl_1 = require("./Crumbl");
const Hasher_1 = require("../Hasher");
const __1 = require("..");
class Uncrumbl {
    constructor(crumbled, slices, verificationHash, signer, isOwner) {
        this.crumbled = crumbled;
        this.slices = slices;
        this.verificationHash = verificationHash;
        this.signer = signer;
        this.isOwner = isOwner;
        this.cipher = new feistel_cipher_1.FPECipher(Obfuscator_1.DEFAULT_HASH_ENGINE, Obfuscator_1.DEFAULT_KEY_STRING, Obfuscator_1.DEFAULT_ROUNDS);
    }
    process() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.doUncrumbl();
        });
    }
    toHTML(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const uncrumbled = yield this.doUncrumbl();
            element.innerText = uncrumbled.toString();
            return uncrumbled;
        });
    }
    doUncrumbl() {
        return __awaiter(this, void 0, void 0, function* () {
            // 1- Parse
            let crumbs = new Array();
            let verificationHash = '';
            try {
                const [vh, crms] = exports.extractData(this.crumbled);
                crumbs = crms;
                verificationHash = vh;
            }
            catch (e) {
                return Promise.reject(e);
            }
            if (verificationHash === '' || verificationHash !== this.verificationHash) {
                return Promise.reject(new Error('incompatible input verification hash with crumbl'));
            }
            // 2- Decrypt crumbs
            const uncrumbs = new Map();
            const indexSet = new Map();
            for (let i = 0; i < crumbs.length; i++) {
                const idx = crumbs[i].index;
                if (!indexSet.has(idx) || indexSet.get(idx)) { // eslint-disable-line @typescript-eslint/strict-boolean-expressions
                    indexSet.set(idx, true);
                }
                if (!this.isOwner && idx === 0 || this.isOwner && idx !== 0) {
                    continue;
                }
                try {
                    const uncrumb = yield Decrypter_1.decrypt(crumbs[i], this.signer);
                    if (!uncrumbs.has(uncrumb.index)) {
                        uncrumbs.set(idx, uncrumb);
                    }
                }
                catch (e) {
                    // NO-OP
                }
            }
            // 3- Add passed uncrumbs
            for (let i = 0; i < this.slices.length; i++) {
                if (!uncrumbs.has(this.slices[i].index)) {
                    uncrumbs.set(this.slices[i].index, this.slices[i]);
                }
            }
            // 4- Determine output
            let hasAllCrumbs = false;
            if (indexSet.size === uncrumbs.size) {
                hasAllCrumbs = true;
            }
            if (this.isOwner && !hasAllCrumbs) {
                return Promise.reject(new Error('missing crumbs to fully uncrumbl as data owner: only partial uncrumbs are returned'));
            }
            if (hasAllCrumbs) {
                // Owner may recover fully-deciphered data
                const collector = new Collector_1.Collector(uncrumbs, indexSet.size, this.verificationHash, crypto_1.DEFAULT_CRYPTO_HASH_ENGINE);
                // 5a- Deobfuscate
                const obfuscated = collector.toObfuscated();
                const obfuscator = new Obfuscator_1.Obfuscator(this.cipher);
                const deobfuscated = obfuscator.unapply(obfuscated);
                // 5b- Unpad
                const unpadded = __1.unpad(deobfuscated);
                return collector.check(Buffer.from(unpadded))
                    .then(isChecked => isChecked ? Promise.resolve(Buffer.from(unpadded)) : Promise.reject(new Error('source has not checked verification hash')));
            }
            else {
                // Trustee could only return his own uncrumbs
                // 5b- Build partial uncrumbs
                let partialUncrumbs = '';
                for (let i = 0; i < indexSet.size; i++) {
                    const maybePartial = uncrumbs.get(i);
                    if (maybePartial !== undefined) {
                        partialUncrumbs += maybePartial.toString();
                    }
                }
                // 6b- Add verification hash prefix and version
                return Buffer.from(this.verificationHash + partialUncrumbs + '.' + Crumbl_1.VERSION);
            }
        });
    }
}
exports.Uncrumbl = Uncrumbl;
/**
 * Extract the verification hash and the crumbs from the passed crumbled string
 *
 * @param {string} crumbl - The full crumbled string
 * @returns the verification hash and the array of crumbs
 */
exports.extractData = (crumbled) => {
    const parts = crumbled.split('.', 2);
    if (parts[1] !== Crumbl_1.VERSION) {
        throw new Error('incompatible version: ' + parts[1]);
    }
    let crumbsStr = parts[0].substr(crypto_1.DEFAULT_HASH_LENGTH);
    const crumbs = new Array();
    while (crumbsStr.length > 0) {
        const nextLen = parseInt(crumbsStr.substr(2, 4), 16);
        const nextCrumb = crumbsStr.substr(0, nextLen + 6);
        const crumb = Crumb_1.toCrumb(nextCrumb);
        crumbs.push(crumb);
        crumbsStr = crumbsStr.substr(nextLen + 6);
    }
    const hasheredSrc = parts[0].substr(0, crypto_1.DEFAULT_HASH_LENGTH);
    const hasher = new Hasher_1.Hasher(crumbs);
    const vh = hasher.unapply(hasheredSrc);
    return [vh, crumbs];
};
/**
 * Extract the hashered prefix from the passed crumbled string
 *
 * @param {string} crumbl - The full crumbled string
 * @returns the hashered prefix
 */
exports.getHasheredSrc = (crumbled) => {
    const parts = crumbled.split('.', 2);
    if (parts[1] !== Crumbl_1.VERSION) {
        throw new Error('incompatible version: ' + parts[1]);
    }
    const hasheredSrc = parts[0].substr(0, crypto_1.DEFAULT_HASH_LENGTH);
    if (hasheredSrc.length !== crypto_1.DEFAULT_HASH_LENGTH) {
        throw new Error('invalid hashered prefix' + hasheredSrc);
    }
    return hasheredSrc;
};
